import base64
import uuid


def uuid4() -> str:
    return str(uuid.uuid4())


def uuid4_no_symbol() -> str:
    return str(uuid.uuid4()).replace('-', '')


# base64 utils

def base64_encode(s):
    return base64.b64encode(s.encode()).decode('utf-8')


def base64_decode(s):
    return base64.b64decode(s).decode('utf-8')



from datetime import datetime
import time

YY_MM_DD_HH_MM_SS_FORMAT = '%Y-%m-%d %H:%M:%S'
YY_MM_DD_FORMAT = '%Y-%m-%d'
# 2021-06-20 14:00:00
STANDARD_DATETIME_FORMAT = YY_MM_DD_HH_MM_SS_FORMAT
# 2021-06-20
STANDARD_DATE_FORMAT = YY_MM_DD_FORMAT


def get_timestamp_ms():
    return int(time.time() * 1000)


def get_timestamp_ns() -> int:
    return time.time_ns()


def calc_duration(st: int) -> int:
    return get_timestamp_ms() - st


def timestamp_to_datetime_str(t, fmt=STANDARD_DATETIME_FORMAT) -> str:
    return format_datetime(datetime.fromtimestamp(t), fmt)


def timestamp_to_date_str(t, fmt=STANDARD_DATE_FORMAT):
    return format_date(datetime.fromtimestamp(t), fmt)


def parse_datetime(s: str, fmt: str = STANDARD_DATETIME_FORMAT):
    return datetime.strptime(s, fmt)


def format_datetime(d: datetime, fmt: str = STANDARD_DATETIME_FORMAT) -> str:
    return d.strftime(fmt)


def parse_date(s: str, fmt: str = STANDARD_DATE_FORMAT):
    return datetime.strptime(s, fmt)


def format_date(d: datetime, fmt: str = STANDARD_DATE_FORMAT):
    return d.strftime(fmt)


def check_datetime(s: str, fmt: str = STANDARD_DATETIME_FORMAT) -> bool:
    try:
        if not s or not isinstance(s, str) or len(s) == 0:
            return False
        parse_datetime(s, fmt)
    except ValueError:
        return False


def check_date(s: str, fmt: str = STANDARD_DATE_FORMAT) -> bool:
    return check_datetime(s, fmt)


def format_duration(st: int) -> str:
    gap = get_timestamp_ms() - st
    if gap < 1000:
        return '{}ms'.format(gap)
    elif gap < 60000:
        return '{}seconds'.format(gap / 1000)
    elif gap < 3600000:
        return '{}minutes'.format(gap / 60000)
    elif gap < 86400000:
        return '{}hours'.format(gap / 3600000)
    else:
        return '{}days'.format(gap / 86400000)



import unittest
from flyers import database
from flyers.database import DatabaseConf


class MyTestCase(unittest.TestCase):
    def test_something(self):
        conf = DatabaseConf(
            database='demo'
        )
        print(conf)
        conn = database.mysql.create_connection(conf)
        database.mysql.run_with(conn,
                                consumer_func=lambda cursor: print(database.mysql.show_mysql_server_version(cursor)))

    if __name__ == '__main__':
        unittest.main()



def check_null(s, name: str):
    if s is None:
        raise ValueError("The '{}' cannot be null".format(name))


def check_blank(s: str, name: str):
    if not str or len(s) == 0:
        raise ValueError("The '{}' cannot be blank".format(name))



import json


def to_json_str(d: dict):
    return json.dumps(d)


def from_json_str(s: str):
    return json.loads(s)


class JsonSerializable(object):

    def to_json_string(self):
        return json.dumps(self, default=lambda obj: obj.__dict__)

    def from_json_string(self, json_string):
        data = json.loads(json_string)

        for key in self.__dict__.keys():
            setattr(self, key, data[key])



import unittest
from flyers import nets


class MyTestCase(unittest.TestCase):
    def test_something(self):
        resp = nets.http_get('http://httpbin.org/get')

        self.assertTrue(resp.status_code, 200)


if __name__ == '__main__':
    unittest.main()



import logging
import sys

__LOGGER_FORMAT = '%(asctime)s [%(threadName)s] %(filename)s %(levelname)s: %(message)s'


def get_level():
    return logger.level


def is_debug():
    return logger.level == logging.DEBUG


def set_critical_level():
    logger.setLevel(logging.CRITICAL)


def set_fatal_level():
    logger.setLevel(logging.FATAL)


def set_error_level():
    logger.setLevel(logging.ERROR)


def set_warning_level():
    logger.setLevel(logging.WARNING)


def set_info_level():
    logger.setLevel(logging.INFO)


def set_debug_level():
    logger.setLevel(logging.DEBUG)


def set_notset_level():
    logger.setLevel(logging.NOTSET)


def get_file_logger(name, filename, fmt=__LOGGER_FORMAT, level=logging.INFO):
    handler = logging.FileHandler(filename, encoding='utf-8')
    handler.setFormatter(logging.Formatter(fmt))
    handler.setLevel(level)
    result = logging.getLogger(name)
    result.setLevel(level)
    result.addHandler(handler)
    return result


def get_console_logger(name, stream=sys.stdout, fmt=__LOGGER_FORMAT, level=logging.INFO):
    handler = logging.StreamHandler(stream=stream)
    handler.setFormatter(logging.Formatter(fmt))
    handler.setLevel(level)
    result = logging.getLogger(name)
    result.setLevel(level)
    result.addHandler(handler)
    return result


# default logger
logger = get_console_logger('flyers')



import smtplib
from email.mime.text import MIMEText
from email.utils import formataddr
from .logs import logger

QQ_SMTP_HOST = "smtp.qq.com"


class EmailClient(object):

    def __init__(self, server_host, sender, sender_name, password):
        self.server_host = server_host
        self.sender = sender
        self.sender_name = sender_name
        self.password = password

    def send(self, subject, body, receivers):
        ret = True
        try:
            msg = MIMEText(body, 'html', 'utf-8')
            msg['From'] = formataddr((self.sender_name, self.sender))
            msg['To'] = formataddr((','.join(receivers), ','.join(receivers)))
            msg['Subject'] = subject

            server = smtplib.SMTP(self.server_host, 25)
            server.starttls()
            server.login(self.sender, self.password)
            server.sendmail(self.sender, receivers, msg.as_string())
            server.quit()
        except Exception as e:
            logger.error(e)
            ret = False
        return ret


class QQEMailClient(EmailClient):

    def __init__(self, sender, sender_name, password):
        super().__init__(QQ_SMTP_HOST, sender, sender_name, password)



from setuptools import setup, find_packages

setup(
    name='flyers',
    version='0.0.3',
    author='pushyzheng',
    author_email='1437876073@qq.com',
    url='https://github.com/pushyzheng',
    description=u'A collections of utils',
    packages=['flyers'],
    install_requires=find_packages()
)



import os
import shutil


class FileSizeUnit(object):
    B = 0
    KB = 1
    MB = 2
    GB = 3
    TB = 4
    PB = 5


def read_file(filename: str) -> str:
    with open(filename, encoding='utf-8') as f:
        return f.read()


def read_lines(filename: str) -> list[str]:
    with open(filename, encoding='utf-8') as f:
        return f.readlines()


def read_and_splitlines(path: str) -> list[str]:
    with open(path, encoding='utf-8') as f:
        return f.read().splitlines()


def write_file(filename: str, data: str):
    with open(filename, 'w+', encoding='utf-8') as f:
        f.write(data)


def update_file(filename: str, func):
    data = read_file(filename)
    new_data = func(data)
    write_file(filename, new_data)


def get_md5(path) -> str:
    files_md5 = os.popen('md5 %s' % path).read().strip()
    file_md5 = files_md5.replace('MD5 (%s) = ' % path, '')
    return file_md5


def copy_dir(src, dst):
    for files in os.listdir(src):
        name = os.path.join(src, files)
        back_name = os.path.join(dst, files)
        if os.path.isfile(name):
            if os.path.isfile(back_name):
                if get_md5(name) != get_md5(back_name):
                    shutil.copy(name, back_name)
            else:
                shutil.copy(name, back_name)
        else:
            if not os.path.isdir(back_name):
                os.makedirs(back_name)
            copy_dir(name, back_name)


def get_file_size(path: str, unit: int) -> float:
    size_units = FileSizeUnit.__dict__
    if unit not in size_units.values():
        raise TypeError('No supports this unit')

    size = os.path.getsize(path)
    for i in range(unit):
        size = size / float(1024)

    return size
